import logging

MAX_PAYLOAD_LENGTH = 25
MAX_TEXT_LENGTH = 100000
ALLOWLIST_TYPE_ID = [str, int]


def to_json(response):
    res_decoded, decoding_err = try_json(response)

    if decoding_err:
        # decoding error, report, but assume unrecoverable
        logging.exception(decoding_err)
        return None

    if not (isinstance(res_decoded, list) or isinstance(res_decoded, dict)):
        logging.error(f"Invalid response type: {type(res_decoded)} (expected: dict)")
        return None

    return res_decoded


def try_json(response):
    """ Decodes response as json, returns decoded and optional error """
    try:
        return response.json(), None
    except ValueError as e:
        return None, e


def check_payload_format(payload: list):
    assert payload, "You should provide a payload"
    assert type(payload) == list, "Payload should be of type list"
    assert (
        len(payload) <= MAX_PAYLOAD_LENGTH
    ), f"Payload should not be longer than {MAX_PAYLOAD_LENGTH} items"
    for item in payload:
        check_item_format(item)


def check_item_format(item: dict):
    assert item.get("id"), "Item does not contain an id key"
    assert (
        type(item.get("id")) in ALLOWLIST_TYPE_ID
    ), "id key should either be a str or a int"

    assert item.get("text"), "Item does not contain a text key"
    assert type(item.get("text")) == str, "text value should be of type str"
    assert (
        len(item.get("text")) <= MAX_TEXT_LENGTH
    ), f"Text should not be longer than {MAX_TEXT_LENGTH} characters"



import codecs
import os
import re

from setuptools import find_packages, setup

NAME = "political_ml"
META_PATH = os.path.join("src", NAME, "__meta__.py")
TESTS_REQUIRE = [
    "pytest>=6.2.5",
    "coverage==5.5",
]
DEV_REQUIRE = TESTS_REQUIRE + [
    "pre-commit==2.12.0",
    "twine~=3.4",
    "build~=0.7",
]

ABOUT = {
    "name": NAME,
    "description": "An API wrapper for a set of API's related to NLP.",
    "url": "https://github.com/wepublic-nl",
    "author": "Wepublic",
    "email": "stakeholderintel@wepublic.nl",
    "license": "Other/Proprietary License",
    "copyright": "All rights reserved",
}


with open("README.md", "r", encoding="utf-8") as fh:
    ABOUT["long_description"] = fh.read()
    ABOUT["long_description_content_type"] = "text/markdown"


HERE = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    """
    Build an absolute path from *parts* and return the contents of the
    resulting file.  Assume UTF-8 encoding.
    """
    with codecs.open(os.path.join(HERE, *parts), "rb", "utf-8") as f:
        return f.read()


META_FILE = read(META_PATH)


def find_meta(meta):
    """
    Extract __*meta*__ from META_FILE.
    """
    meta_match = re.search(
        r"^__{meta}__ = ['\"]([^'\"]*)['\"]".format(meta=meta), META_FILE, re.M
    )
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError("Unable to find __{meta}__ string.".format(meta=meta))


setup(
    **ABOUT,
    version=find_meta("version"),
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    python_requires=">= 3.8",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    install_requires=[
        "requests>=2.25.1",
    ],
    extras_require={"test": TESTS_REQUIRE, "dev": DEV_REQUIRE},
    project_urls={
        "Source": "https://github.com/wepublic-nl/" + NAME,
    },
)



import logging
import uuid
from os.path import join

import requests

from political_ml.__meta__ import __version__
from political_ml.utils import check_payload_format, to_json

logger = logging.getLogger(__name__)


class _BaseClient:
    def __init__(self, endpoint, token):
        self._endpoint = endpoint
        self._token = token

    def exec_request(self, method, url, **kwargs):
        """ Executes a request, assigning a unique id beforehand and throwing on 4xx / 5xx """
        reqid = str(uuid.uuid4())

        logging.debug(
            f"{self.__class__.__qualname__} -> {method.upper()} {url} {reqid=}"
        )

        # requests.post / requests.get / ...
        method_exec = getattr(
            requests, method.lower()
        )  # retrieves the right method from requests

        headers = self._build_headers()
        response = method_exec(url, headers=headers, **kwargs)

        status_code = response.status_code
        content_length = len(response.content or "")
        logging.debug(
            f"{self.__class__.__qualname__} <- {status_code} {content_length} {reqid=}"
        )

        if response.status_code >= 300:
            logging.debug(response.json())

        # raise by default to halt further exec and bubble
        response.raise_for_status()

        return to_json(response)

    def build_url(self, *paths):
        return join(self._endpoint, *paths)

    def _build_headers(self):
        return {
            "Accept": "application/json",
            "User-Agent": f"Political ML Python package {__version__}",
            "Authorization": f"Bearer {self._token}",
        }


class NerClient(_BaseClient):
    def ner(self, payload) -> list:
        check_payload_format(payload)
        response = self.exec_request(
            method="POST",
            url=self.build_url("api/v1/ner"),
            json=payload,
        )
        return response or []


class CategoriseClient(_BaseClient):
    def categorise(self, payload) -> list:
        check_payload_format(payload)
        response = self.exec_request(
            method="POST",
            url=self.build_url("api/v1/categorise"),
            json=payload,
        )
        return response or []


class ArticleExtractionClient(_BaseClient):
    def by_url(self, url: str, clean: bool = True) -> dict:
        assert type(url) == str, "Url must be a string"
        payload = {"url": url}
        response = self.exec_request(
            method="POST",
            url=self.build_url(f"extract/url?clean={clean}"),
            json=payload,
        )
        return response or {}

    def by_html(self, html: str, clean: bool = True) -> dict:
        assert type(html) == str, "Html must be a string"
        payload = {"html": html}
        response = self.exec_request(
            method="POST",
            url=self.build_url(f"extract/html?clean={clean}"),
            json=payload,
        )
        return response or {}
